/*
 * This file is part of Squirrels.
 * Copyright (C) 2009 Squirrels Develop Team.
 *
 * Squirrels is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License any later version.
 *
 * Squirrels is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.	 See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with Squirrels. If not, see <http://www.gnu.org/licenses/>.
 */
package squirrels.ecshop.action.images;

import nuts.core.lang.NumberUtils;
import nuts.core.lang.StringUtils;

import org.apache.commons.vfs2.FileContent;
import org.apache.commons.vfs2.FileObject;

import squirrels.ecshop.model.bean.Goods;
import squirrels.ecshop.model.dao.GoodsDAO;


/**
 * GoodsImageAction
 */
@SuppressWarnings("serial")
public class GoodsImageAction extends AbstractImageDownloadAction {

	private String goodsId;

	/**
	 * @return the goodsId
	 */
	public String getGid() {
		return goodsId;
	}

	/**
	 * @param goodsId the goodsId to set
	 */
	public void setGid(String goodsId) {
		this.goodsId = goodsId;
	}

	/**
	 * @return the imageName
	 */
	public String getGin() {
		return imageName;
	}

	/**
	 * @param imageName the imageName to set
	 */
	public void setGin(String imageName) {
		this.imageName = imageName;
	}

	/**
	 * execute
	 * @return result name
	 * @throws Exception if an error occurs
	 */
	public String execute() throws Exception {
		Long gid = NumberUtils.toLong(goodsId);
		if (gid != null) {
			if (StringUtils.isEmpty(imageName)) {
				GoodsDAO gdao = new GoodsDAO(getDataAccessSession());
				Goods g = gdao.selectByPrimaryKey(gid);
				if (g != null) {
					String[] ss = StringUtils.split(g.getThumbs());
					if (ss != null && ss.length > 0) {
						imageName = ss[0];
					}
				}
			}
				
			if (StringUtils.isNotEmpty(imageName)) {
				FileObject fo = utils().getGoodsImagesPath(gid);
				if (fo.exists()) {
					FileObject fi = fo.resolveFile(imageName);
					if (fi.exists()) {
						FileContent fc = fi.getContent();
						imageSize = (int)fc.getSize();
						imageStream = fc.getInputStream();
						return success();
					}
				}
			}
		}
		return NOIMG;
	}
}